library(ggplot2)
library(purrr)
library(readr)
library(tidyr)
library(tikzDevice)
suppressMessages({
library(dplyr)
library(igraph)
})

#####
print("Loading scripts")

source("scripts/initialization.R")
source("scripts/properties.R")
source("scripts/heuristics.R")

options(show.error.locations = TRUE)


#####
print("Computing initial DAGs")

n <- 128
n_even <- (n %/% 2) * 2
n_odd <- ((n - 1) %/% 2) * 2 + 1
n_power <- 2 ^ as.integer(log2(n))
n_square <- as.integer(sqrt(n)) ^ 2
n_triangular <- ifelse(sum(seq_len(sqrt(2 * n))) <= n,
                       sum(seq_len(sqrt(2 * n))),
                       sum(seq_len(sqrt(2 * n) - 1 / 2)))
DAG <- list(empty = DAG_empty(n),
            complete = DAG_complete(n),
            chain = DAG_transitive_reduce(DAG_complete(n)),
            outtree = DAG_outtree(n_power - 1),
            intree = DAG_reverse(DAG_outtree(n_power - 1)),
            comb = DAG_comb(n_odd),
            combR = DAG_reverse(DAG_comb(n_odd)),
            bipartite = DAG_bipartite(n_even),
            square = DAG_square(n_square),
            triangular = DAG_triangular(n_triangular))
DAG <- map(DAG, graph_from_adjacency_matrix)


#####
print("Generating schedules")
pb <- txtProgressBar(style = 3)

df <- list()
method <- c(HEFT, MinMin, HCPT)
method_names <- c("HEFT", "MinMin", "HCPT")
max_procs <- sqrt(2 * n)
for (nb_processors in seq_len(max_procs))
  for (i in seq_along(DAG)) {
    setTxtProgressBar(pb, (i - 1 + (nb_processors - 1) * length(DAG)) /
                          (max_procs * length(DAG)))
    res <- map(method, ~ .(nb_processors, DAG[[i]])) %>%
      map(~ select(., end) %>% summarize(max(end)) %>% unlist)
    names(res) <- method_names
    df[[length(df) + 1]] <- c(list(nb_proc = nb_processors, dag = i), res)
  }
df <- bind_rows(df)
setTxtProgressBar(pb, 1)
close(pb)


#####
print("Saving data")

dir.create(path = "data", showWarnings = FALSE)
write_rds(df, "data/1_special_schedule.rds", "xz", compression = 9L)
# df <- read_rds("data/1_special_schedule.rds")


#####
print("Generating figure")

dag_names <- 1:10
dag_lab <- c("$D_{\\textnormal{empty}}$",
             "$D_{\\textnormal{complete}}/D_{\\textnormal{chain}}$",
             "$D_{\\textnormal{chain}}{\\textnormal{chain}}$",
             "$D_{\\textnormal{out-tree}}/D_{\\textnormal{in-tree}}$",
             "$D_{\\textnormal{int-ree}}$", "$D_{\\textnormal{comb}}$",
             "$D_{\\textnormal{comb}}^R$",
             "$D_{\\textnormal{bipartite}}$",
             "$D_{\\textnormal{square}}$",
             "$D_{\\textnormal{triangular}}$")

p <- df %>%
  filter(!dag %in% c(3, 5)) %>%
  mutate(dag = factor(dag, levels = dag_names, labels = dag_lab)) %>%
  gather(method, mks, -nb_proc, -dag) %>%
  mutate(method = factor(method, levels = method_names)) %>%
  ggplot(aes(x = nb_proc, y = mks, colour = method, shape = method,
             size = as.numeric(method))) +
  geom_point() +
  geom_line() +
  facet_wrap(~ dag, ncol = 4) +
  scale_color_hue(name = "Heuristic~~", l = c(90, 45, 0)) +
  scale_shape(name = "Heuristic~~") +
  scale_size(guide = FALSE, range = c(0.5, 2), trans = "reciprocal") +
  scale_x_continuous(name = "Processor number $|P|$") +
  scale_y_continuous(name = "Makespan") +
  theme(legend.position = "bottom")

dir.create(path = "plots", showWarnings = FALSE)
tikz(file = "plots/1_special_schedule.tex", width = 6, height = 4)
print(p)
dev.off()
