library(ggplot2)
library(readr)
library(tidyr)
library(tikzDevice)
suppressMessages({
library(dplyr)
library(igraph)
})

#####
print("Loading scripts")

source("scripts/generation.R")
source("scripts/properties.R")

options(show.error.locations = TRUE)


#####
print("Generating Erdos-Renyi DAGs and measuring properties")
pb <- txtProgressBar(style = 3)

set.seed(1)
MC <- 300
NB_NODES <- 100
min_size <- 10
max_size <- 200
PROBA <- 0.15
result <- list()
for (i in 1:MC) {
  p <- runif(1)
  setTxtProgressBar(pb, (i - 1) / (MC + max_size - min_size))
  mat <- erdos_renyi(NB_NODES, p)
  result[[length(result) + 1]] <- c(DAG_prop(mat),
                                    list(proba = p, iter = i))
}
for (n in min_size:max_size) {
  setTxtProgressBar(pb, (n - min_size + MC) /
                        (MC + max_size - min_size))
  mat <- erdos_renyi(n, PROBA)
  result[[length(result) + 1]] <- c(DAG_prop(mat),
                                    list(proba = PROBA,
                                         iter = MC + n - min_size + 1))
}
result <- bind_rows(result)
setTxtProgressBar(pb, 1)
close(pb)


#####
print("Saving data")

dir.create(path = "data", showWarnings = FALSE)
write_rds(result, "data/2_erdos_analysis.rds", "xz", compression = 9L)
# result <- read_rds("data/2_erdos_analysis.rds")


#####
print("Generating figure")

prop_names <- c("length", "m_edges",  "m_edges_red", "mass",
                "shape_mean", "shape_sd", "deg_max_red", "deg_sd_red")
prop_lab <- c("$\\textnormal{len}$", "$m$", "$m(D^T)$",
              "$\\textnormal{mass}$",
              "$\\textnormal{sh}^{\\textnormal{mean}}$",
              "$\\textnormal{sh}^{\\textnormal{CV}}$",
              "$\\textnormal{deg}^{\\textnormal{max}}(D^T)$",
              "$\\textnormal{deg}^{\\textnormal{CV}}(D^T)$")

tidy_data <- result %>%
  mutate(shape_sd = shape_sd / shape_mean) %>%
  mutate(deg_sd_red = deg_sd_red / deg_mean_red) %>%
  gather(prop, value, -n_nodes, -proba, -iter) %>%
  filter(prop %in% prop_names) %>%
  group_by(prop) %>%
  mutate(theo = ifelse(prop == "length", proba * n_nodes, NA)) %>%
  mutate(theo = ifelse(prop == "m_edges", proba * n_nodes *
                                          (n_nodes - 1) / 2, theo)) %>%
  mutate(theo = ifelse(prop == "m_edges_red",
                       (n_nodes - 1) / proba - (1 - proba ^ 2) / proba ^ 3 *
                       (1 - (1 - proba ^ 2) ^ (n_nodes - 1)), theo)) %>%
  mutate(theo = ifelse(prop == "shape_mean" & 1 / proba < max(value),
                       1 / proba, theo)) %>%
  ungroup() %>%
  mutate(prop = factor(prop, levels = prop_names, labels = prop_lab))

p <- tidy_data %>%
  filter(iter <= MC) %>%
  ggplot(aes(x = proba, y = value)) +
  geom_line(aes(y = theo), color = "red", size = 3) +
  geom_point(size = 0.5) +
  geom_smooth(method = "lm", formula = y ~ splines::bs(x, 10)) +
  facet_wrap(~ prop, scales = "free_y", ncol = 4) +
  scale_x_continuous(name = "Probability $p$",
                     labels = function(x) format(x, drop0trailing = TRUE)) +
  scale_y_continuous(name = "Value") +
  theme(legend.position = "bottom")

dir.create(path = "plots", showWarnings = FALSE)
tikz(file = "plots/2_erdos_analysis_proba.tex", width = 6, height = 3)
print(p)
dev.off()

dir.create(path = "plots", showWarnings = FALSE)
tikz(file = "plots/2_erdos_analysis_proba_europar.tex", width = 4.8,
     height = 2.5)
print(p + theme(axis.text.x = element_text(angle = 45, hjust = 1)))
dev.off()

p <- tidy_data %>%
  filter(iter > MC) %>%
  ggplot(aes(x = n_nodes, y = value)) +
  geom_line(aes(y = theo), color = "red", size = 3) +
  geom_point(size = 0.5) +
  geom_smooth(method = "lm", formula = y ~ splines::bs(x, 4)) +
  facet_wrap(~ prop, scales = "free_y", ncol = 4) +
  scale_x_continuous(name = "Number of vertices $n$") +
  scale_y_continuous(name = "Value") +
  theme(legend.position = "bottom")

dir.create(path = "plots", showWarnings = FALSE)
tikz(file = "plots/2_erdos_analysis_node.tex", width = 6, height = 3)
print(p)
dev.off()
