library(ggplot2)
library(readr)
library(tidyr)
library(tikzDevice)
suppressMessages({
library(dplyr)
library(igraph)
library(rPython)
})

#####
print("Loading scripts")

source("scripts/properties.R")

python.load("scripts/recursive.py")

options(show.error.locations = TRUE)


#####
print("Precomputation with recursive method (takes a few minutes)")
min_size <- 10
max_size <- 200
python.call(py.foo = "init", n = max_size, seed = 1)


#####
print("Generating uniform DAGs and measuring properties")
pb <- txtProgressBar(style = 3)

result <- list()
for (n in min_size:max_size) {
  setTxtProgressBar(pb, (n - min_size) / (max_size - min_size))
  edge_list <- python.call(py.foo = "generateUniformDAG", n = n)
  edge_list <- do.call(rbind, edge_list)
  dag <- graph_from_edgelist(edge_list + 1)
  mat <- as_adjacency_matrix(dag) %>% as.matrix
  result[[length(result) + 1]] <- c(DAG_prop(mat))
}
result <- bind_rows(result)
setTxtProgressBar(pb, 1)
close(pb)


#####
print("Saving data")

dir.create(path = "data", showWarnings = FALSE)
write_rds(result, "data/3_recursive_analysis.rds", "xz", compression = 9L)
# result <- read_rds("data/3_recursive_analysis.rds")


#####
print("Generating figure")

prop_names <- c("length", "m_edges",  "m_edges_red", "mass",
                "shape_mean", "shape_sd", "deg_max_red", "deg_sd_red")
prop_lab <- c("$\\textnormal{len}$", "$m$", "$m(D^T)$",
              "$\\textnormal{mass}$",
              "$\\textnormal{sh}^{\\textnormal{mean}}$",
              "$\\textnormal{sh}^{\\textnormal{CV}}$",
              "$\\textnormal{deg}^{\\textnormal{max}}(D^T)$",
              "$\\textnormal{deg}^{\\textnormal{CV}}(D^T)$")

tidy_data <- result %>%
  mutate(shape_sd = shape_sd / shape_mean) %>%
  mutate(deg_sd_red = deg_sd_red / deg_mean_red) %>%
  gather(prop, value, -n_nodes) %>%
  filter(prop %in% prop_names) %>%
  group_by(prop) %>%
  mutate(theo = ifelse(prop == "length", n_nodes / 1.488, NA)) %>%
  mutate(theo = ifelse(prop == "m_edges", n_nodes ^ 2 / 4, theo)) %>%
#  mutate(theo = ifelse(prop == "mass", log(n_nodes) ^ 4 / n_nodes, theo)) %>%
  mutate(theo = ifelse(prop == "shape_mean", 1.488, theo)) %>%
  ungroup() %>%
  mutate(prop = factor(prop, levels = prop_names, labels = prop_lab))

p <- tidy_data %>%
  ggplot(aes(x = n_nodes, y = value)) +
  geom_line(aes(y = theo), color = "red", size = 3) +
  geom_point(size = 0.5) +
  geom_smooth(method = "lm", formula = y ~ splines::bs(x, 4)) +
  facet_wrap(~ prop, scales = "free_y", ncol = 4) +
  scale_x_continuous(name = "Number of vertices $n$") +
  scale_y_continuous(name = "Value") +
  theme(legend.position = "bottom")

dir.create(path = "plots", showWarnings = FALSE)
tikz(file = "plots/3_recursive_analysis.tex", width = 6, height = 3)
print(p)
dev.off()

dir.create(path = "plots", showWarnings = FALSE)
tikz(file = "plots/3_recursive_analysis_europar.tex", width = 4.8, height = 2.5)
print(p + theme(axis.text.x = element_text(angle = 45, hjust = 1)))
dev.off()
