library(ggplot2)
library(readr)
library(tidyr)
library(tikzDevice)
suppressMessages({
library(dplyr)
library(igraph)
})

#####
print("Loading scripts")

source("scripts/generation.R")
source("scripts/properties.R")

options(show.error.locations = TRUE)


#####
print("Generating random orders DAGs and measuring properties")
pb <- txtProgressBar(style = 3)

set.seed(1)
MC <- 30
NB_NODES <- 100
K <- 3
orders <- 2:15
sizes <- 10:200
iter <- 0
result <- list()
for (k in orders)
  for (i in 1:MC) {
    setTxtProgressBar(pb, iter / (length(orders) * MC + length(sizes)))
    iter <- iter + 1
    mat <- random_orders(NB_NODES, k)
    result[[length(result) + 1]] <- c(DAG_prop(mat),
                                      list(k = k, iter = iter))
  }
for (n in sizes) {
  setTxtProgressBar(pb, iter / (length(orders) * MC + length(sizes)))
  iter <- iter + 1
  mat <- random_orders(n, K)
  result[[length(result) + 1]] <- c(DAG_prop(mat),
                                    list(k = K, iter = iter))
}
result <- bind_rows(result)
setTxtProgressBar(pb, 1)
close(pb)


#####
print("Saving data")

dir.create(path = "data", showWarnings = FALSE)
write_rds(result, "data/4_poset_analysis.rds", "xz", compression = 9L)
# result <- read_rds("data/4_poset_analysis.rds")


#####
print("Generating figure")

prop_names <- c("length", "m_edges",  "m_edges_red", "mass",
                "shape_mean", "shape_sd", "deg_max_red", "deg_sd_red")
prop_lab <- c("$\\textnormal{len}$", "$m$", "$m(D^T)$",
              "$\\textnormal{mass}$",
              "$\\textnormal{sh}^{\\textnormal{mean}}$",
              "$\\textnormal{sh}^{\\textnormal{CV}}$",
              "$\\textnormal{deg}^{\\textnormal{max}}(D^T)$",
              "$\\textnormal{deg}^{\\textnormal{CV}}(D^T)$")

tidy_data <- result %>%
  mutate(shape_sd = shape_sd / shape_mean) %>%
  mutate(deg_sd_red = deg_sd_red / deg_mean_red) %>%
  gather(prop, value, -n_nodes, -k, -iter) %>%
  filter(prop %in% prop_names) %>%
  group_by(prop) %>%
  mutate(theo = ifelse(prop == "length", n_nodes ^ (1 / k), NA)) %>%
  mutate(theo = ifelse(prop == "shape_mean",
                       n_nodes ^ (1 - 1 / k), theo)) %>%
  ungroup() %>%
  mutate(prop = factor(prop, levels = prop_names, labels = prop_lab))

p <- tidy_data %>%
  filter(iter <= length(orders) * MC) %>%
  ggplot(aes(x = factor(k), y = value)) +
  geom_line(aes(x = k - 1, y = theo), color = "red", size = 3) +
  geom_boxplot(outlier.size = 0.5) +
  facet_wrap(~ prop, scales = "free_y", ncol = 4) +
  scale_x_discrete(name = "Number of permutations $K$",
                   labels = ifelse(orders %% 2 == 0, orders, "")) +
  scale_y_continuous(name = "Value") +
  theme(legend.position = "bottom")

dir.create(path = "plots", showWarnings = FALSE)
tikz(file = "plots/4_poset_analysis_perm.tex", width = 6, height = 3)
print(p)
dev.off()

dir.create(path = "plots", showWarnings = FALSE)
tikz(file = "plots/4_poset_analysis_perm_europar.tex", width = 4.8,
     height = 2.5)
print(p + theme(axis.text.x = element_text(angle = 90, hjust = 1,
                                           vjust = 0.5)))
dev.off()

p <- tidy_data %>%
  filter(iter > length(orders) * MC) %>%
  ggplot(aes(x = n_nodes, y = value)) +
  geom_line(aes(y = theo), color = "red", size = 3) +
  geom_point(size = 0.5) +
  geom_smooth(method = "lm", formula = y ~ splines::bs(x, 4)) +
  facet_wrap(~ prop, scales = "free_y", ncol = 4) +
  scale_x_continuous(name = "Number of vertices $n$") +
  scale_y_continuous(name = "Value") +
  theme(legend.position = "bottom")

dir.create(path = "plots", showWarnings = FALSE)
tikz(file = "plots/4_poset_analysis_node.tex", width = 6, height = 3)
print(p)
dev.off()
