library(ggplot2)
library(readr)
library(tidyr)
library(tikzDevice)
suppressMessages({
library(dplyr)
library(igraph)
})

#####
print("Loading scripts")

source("scripts/generation.R")
source("scripts/properties.R")

options(show.error.locations = TRUE)


#####
print("Generating layer-by-layer DAGs and measuring properties")
pb <- txtProgressBar(style = 3)

set.seed(1)
MC <- 300
NB_NODES <- 100
min_size <- 10
max_size <- 200
LAYER <- 10
PROBA <- 0.5
iter <- 0
result <- list()
for (i in 1:MC) {
  setTxtProgressBar(pb, iter / (2 * MC + max_size - min_size))
  iter <- iter + 1
  proba <- runif(1)
  mat <- layer_by_layer(NB_NODES, LAYER, proba)
  result[[length(result) + 1]] <- c(DAG_prop(mat),
                                    list(layer = LAYER, iter = iter,
                                         proba = proba))
}
for (i in 1:MC) {
  setTxtProgressBar(pb, iter / (2 * MC + max_size - min_size))
  iter <- iter + 1
  layer <- runif(1, min = log(1), max = log(NB_NODES + 1)) %>%
    exp %>% as.integer
  mat <- layer_by_layer(NB_NODES, layer, PROBA)
  result[[length(result) + 1]] <- c(DAG_prop(mat),
                                    list(layer = layer, iter = iter,
                                         proba = PROBA))
}
for (n in min_size:max_size) {
  setTxtProgressBar(pb, iter / (2 * MC + max_size - min_size))
  iter <- iter + 1
  layer <- as.integer(sqrt(n) + 0.5)
  mat <- layer_by_layer(n, layer, PROBA)
  result[[length(result) + 1]] <- c(DAG_prop(mat),
                                    list(layer = layer, iter = iter,
                                         proba = PROBA))
}
result <- bind_rows(result)
setTxtProgressBar(pb, 1)
close(pb)


#####
print("Saving data")

dir.create(path = "data", showWarnings = FALSE)
write_rds(result, "data/5_layer_analysis.rds", "xz", compression = 9L)
# result <- read_rds("data/5_layer_analysis.rds")


#####
print("Generating figure")

prop_names <- c("length", "m_edges",  "m_edges_red", "mass",
                "shape_mean", "shape_sd", "deg_max_red", "deg_sd_red")
prop_lab <- c("$\\textnormal{len}$", "$m$", "$m(D^T)$",
              "$\\textnormal{mass}$",
              "$\\textnormal{sh}^{\\textnormal{mean}}$",
              "$\\textnormal{sh}^{\\textnormal{CV}}$",
              "$\\textnormal{deg}^{\\textnormal{max}}(D^T)$",
              "$\\textnormal{deg}^{\\textnormal{CV}}(D^T)$")

tidy_data <- result %>%
  mutate(shape_sd = shape_sd / shape_mean) %>%
  mutate(deg_sd_red = deg_sd_red / deg_mean_red) %>%
  gather(prop, value, -n_nodes, -layer, -iter, -proba) %>%
  filter(prop %in% prop_names) %>%
  group_by(prop) %>%
  mutate(theo = ifelse(prop == "length", layer, NA)) %>%
  mutate(theo = ifelse(prop == "m_edges",
                       n_nodes * (1 - 1 / layer) *
                       (proba * (n_nodes / 2 - 1) + 1), theo)) %>%
  mutate(theo = ifelse(prop == "m_edges_red",
                       proba * (layer - 1) * (n_nodes / layer) ^ 2 +
                       (1 - proba) * n_nodes * (1 - 1 / layer), theo)) %>%
  mutate(theo = ifelse(prop == "shape_mean", n_nodes / layer, theo)) %>%
  ungroup() %>%
  mutate(layer_mass = ifelse(prop == "mass", 20, NA)) %>%
  mutate(prop = factor(prop, levels = prop_names, labels = prop_lab))

p <- tidy_data %>%
  filter(iter <= MC) %>%
  ggplot(aes(x = proba, y = value)) +
  geom_line(aes(y = theo), color = "red", size = 3) +
  geom_point(size = 0.5) +
  geom_smooth(method = "lm", formula = y ~ splines::bs(x, 4)) +
  facet_wrap(~ prop, scales = "free_y", ncol = 4) +
  scale_x_continuous(name = "Probability $p$",
                     labels = function(x) format(x, drop0trailing = TRUE)) +
  scale_y_continuous(name = "Value") +
  theme(legend.position = "bottom")

dir.create(path = "plots", showWarnings = FALSE)
tikz(file = "plots/5_layer_analysis_proba.tex", width = 6, height = 3)
print(p)
dev.off()

p <- tidy_data %>%
  filter(iter > MC & iter <= 2 * MC) %>%
  ggplot(aes(x = layer, y = value)) +
  geom_line(aes(y = theo), color = "red", size = 3) +
  geom_vline(aes(xintercept = layer_mass), color = "red", size = 3) +
  geom_point(size = 0.5) +
  geom_smooth(method = "lm", formula = y ~ splines::bs(x, 5)) +
  facet_wrap(~ prop, scales = "free_y", ncol = 4) +
  scale_x_log10(name = "Number of layers $k$") +
  scale_y_continuous(name = "Value") +
  annotation_logticks(sides = "b") +
  theme(legend.position = "bottom")

dir.create(path = "plots", showWarnings = FALSE)
tikz(file = "plots/5_layer_analysis_layer.tex", width = 6, height = 3)
print(p)
dev.off()

dir.create(path = "plots", showWarnings = FALSE)
tikz(file = "plots/5_layer_analysis_layer_europar.tex", width = 4.8,
     height = 2.5)
print(p)
dev.off()

p <- tidy_data %>%
  filter(iter > 2 * MC) %>%
  ggplot(aes(x = n_nodes, y = value)) +
  geom_line(aes(y = theo), color = "red", size = 3) +
  geom_point(size = 0.5) +
  geom_smooth(method = "lm", formula = y ~ splines::bs(x, 4)) +
  facet_wrap(~ prop, scales = "free_y", ncol = 4) +
  scale_x_continuous(name = "Number of vertices $n$") +
  scale_y_continuous(name = "Value") +
  theme(legend.position = "bottom")

dir.create(path = "plots", showWarnings = FALSE)
tikz(file = "plots/5_layer_analysis_node.tex", width = 6, height = 3)
print(p)
dev.off()
