library(ggplot2)
library(purrr)
library(readr)
library(tidyr)
library(tikzDevice)
suppressMessages({
library(dplyr)
library(igraph)
library(rPython)
})

#####
print("Loading scripts")

source("scripts/generation.R")
source("scripts/heuristics.R")

python.load("scripts/recursive.py")

options(show.error.locations = TRUE)


#####
print("Precomputation with recursive method (takes a few minutes)")
max_size <- 100
python.call(py.foo = "init", n = max_size, seed = 1)


#####
print("Computing DAGs and schedules")
pb <- txtProgressBar(style = 3)

set.seed(1)
MC <- 300
n <- 100
nb_processors <- c(2, 3, 5, 7, 10)
method <- c(HEFT, MinMin, HCPT)
method_names <- c("HEFT", "MinMin", "HCPT")
df <- list()
for (i in 1:MC) {
  setTxtProgressBar(pb, (i - 1) / MC)
  edge_list <- python.call(py.foo = "generateUniformDAG", n = n)
  edge_list <- do.call(rbind, edge_list)
  dag <- graph_from_edgelist(edge_list + 1)
  mat <- as_adjacency_matrix(dag) %>% as.matrix

  DAG <- list(erdos = erdos_renyi(n, 0.15),
              rec = mat,
              perm = random_orders(n, 3),
              layer = layer_by_layer(n, sqrt(n), 0.5))
  DAG <- map(DAG, graph_from_adjacency_matrix)

  for (proc in nb_processors)
    for (j in seq_along(DAG)) {
      costs <- as.integer(runif(n, min = 1, max = 1))
      res <- map(method, ~ .(proc, DAG[[j]], costs) %>%
                           summarize(max(end)) %>% unlist)
      names(res) <- method_names
      df[[length(df) + 1]] <- c(list(iter = i, dag = j,
                                     nb_proc = proc), res)
    }
}
df <- bind_rows(df)
setTxtProgressBar(pb, 1)
close(pb)


#####
print("Saving data")

dir.create(path = "data", showWarnings = FALSE)
write_rds(df, "data/6_heuristics_comparison.rds", "xz", compression = 9L)
# df <- read_rds("data/6_heuristics_comparison.rds")


#####
print("Generating figure")

proc_names <- paste0("$|P|=", nb_processors, "$")
dag_lab <- c("Erd.-R.",
             "Recur.",
             "R. ord.",
             "Layered")
dag_names <- seq_along(dag_lab)

p <- df %>%
  mutate(nb_proc = factor(nb_proc, levels = nb_processors,
                          labels = proc_names)) %>%
  mutate(dag = factor(dag, levels = dag_names, labels = dag_lab)) %>%
  mutate(min_mks = pmin(MinMin, HEFT, HCPT)) %>%
  gather(method, mks, -iter, -dag, -nb_proc, -min_mks) %>%
  mutate(method = factor(method, levels = method_names)) %>%
  ggplot(aes(x = method, y = mks - min_mks, colour = method)) +
  geom_boxplot(outlier.size = 0.5) +
  facet_grid(dag ~ nb_proc) +
  scale_color_hue(guide = FALSE, l = c(90, 45, 0)) +
  scale_x_discrete(name = "Heuristic") +
  scale_y_continuous(name = "Absolute makespan difference") +
  theme(legend.position = "bottom") +
  coord_flip()

dir.create(path = "plots", showWarnings = FALSE)
tikz(file = "plots/6_heuristics_comparison.tex", width = 6, height = 3.5)
print(p)
dev.off()

dir.create(path = "plots", showWarnings = FALSE)
tikz(file = "plots/6_heuristics_comparison_europar.tex", width = 4.8,
     height = 3)
print(p)
dev.off()
