suppressMessages({
library(dplyr)
library(igraph)
})

erdos_renyi <- function(nb_nodes, proba) {
  require(igraph)
  mat <- matrix(runif(nb_nodes * nb_nodes), nrow = nb_nodes,
                ncol = nb_nodes)
  mat <- upper.tri(mat) * (mat <= proba) * 1
  mat
  #graph_from_adjacency_matrix(adjmatrix = mat, mode = "directed")
}

# Alternative edge-based version
erdos_renyi_edges <- function(nb_nodes, proba) {
  require(dplyr)
  require(igraph)
  edges <- expand.grid(src = seq_len(nb_nodes),
                       dst = seq_len(nb_nodes)) %>%
    filter(src < dst) %>%
    filter(runif(n()) < proba) %>%
    as.matrix
  dag <- graph_from_edgelist(edges)
  add.vertices(dag, nb_nodes - gorder(dag)) %>%
    as_adjacency_matrix
}

random_orders <- function(nb_nodes, nb_orders) {
  require(dplyr)
  require(igraph)
  keep_edges <- expand.grid(src = seq_len(nb_nodes),
                            dst = seq_len(nb_nodes)) %>%
    mutate(dir = src < dst) %>%
    select(dir)
  ords <- list()
  for (i in seq(nb_orders)) {
    ord <- sample(nb_nodes)
    edges <- expand.grid(src = ord, dst = ord) %>%
      cbind(keep_edges) %>%
      filter(dir) %>%
      select(-dir)
    ords[[length(ords) + 1]] <- edges
  }
  edges <- bind_rows(ords) %>%
    group_by(src, dst) %>%
    summarise(n = n()) %>%
    filter(n == nb_orders) %>%
    select(-n) %>%
    as.matrix
  dag <- graph_from_edgelist(edges)
  add.vertices(dag, nb_nodes - gorder(dag)) %>%
    as_adjacency_matrix %>%
    as.matrix
}

layer_by_layer <- function(nb_nodes, nb_layers, edge_proba) {
  require(dplyr)
  require(igraph)
  # Vertices distributed with balls into bins approach
  # (at least one vertex per layer)
  layers <- runif(nb_nodes - nb_layers, min = 1, max = nb_layers + 1) %>%
    as.integer %>%
    c(1:nb_layers) %>%
    sort
  # Proba to keep any edge between two consecutive layers
  edges <- NULL
  other_edges <- NULL
  for (i in 1:nb_nodes)
    if (layers[i] != 1) {
      # At least one parent for each vertex
      preds <- which(layers == layers[i] - 1)
      pred <- preds[sample(seq_along(preds), 1)]
      edges[[length(edges) + 1]] <- c(pred, i)
      # All other predecessors are candidate
      all_preds <- which(layers < layers[i])
      all_preds <- setdiff(all_preds, pred)
      if (length(all_preds) >= 1)
        other_edges[[length(other_edges) + 1]] <- cbind(all_preds, i)
    }
  if (!is.null(edges))
    edges <- do.call(rbind, edges)
  if (!is.null(other_edges)) {
    other_edges <- do.call(rbind, other_edges)
    other_edges <- other_edges[runif(nrow(other_edges)) < edge_proba,]
  }
  # Merge and matrix build
  if (!is.null(edges) | !is.null(other_edges))
    dag <- graph_from_edgelist(rbind(edges, other_edges))
  else
    dag <- make_empty_graph(nb_nodes)
  add.vertices(dag, nb_nodes - gorder(dag)) %>%
    as_adjacency_matrix %>%
    as.matrix
}
