MinMin <- function(nb_processors, dag, costs = rep(1, gorder(dag)),
                   maxmin = FALSE) {
  require(igraph)
  require(purrr)
  # Get infos
  adj <- as_adjacency_matrix(dag) %>% as.matrix
  schedule <- data.frame(task = seq_along(costs), proc = NA,
                         start = NA, end = Inf)
  scheduled <- rep(FALSE, length(costs))
  terminated <- rep(FALSE, length(costs))
  curr_time <- 0
  proc_ready <- rep(0, nb_processors)
  while (any(!scheduled)) {
    # find next relevant time
    relevant_next_times <- sort(unique(pmax(proc_ready, curr_time)))
    for (curr_time in relevant_next_times)
      if (any(schedule$end == curr_time & !terminated) ||
          any(colSums(adj) == 0 & !scheduled))
        # either a task is finishing or a task is ready
        break
    # compute which tasks has just finished and remove dependencies
    # that are met
    finishing_tasks <- which(schedule$end == curr_time & !terminated)
    if (length(finishing_tasks) != 0) {
      adj[finishing_tasks,] <- 0
      terminated[finishing_tasks] <- TRUE
    }
    # compute which task has a minimum cost among the ready ones
    ready_tasks <- which(colSums(adj) == 0 & !scheduled)
    if (length(ready_tasks) != 0) {
      if (!maxmin)
        task <- ready_tasks[which.min(costs[ready_tasks])]
      else
        task <- ready_tasks[which.max(costs[ready_tasks])]
      # schedule it on a processor available at current time
      ready_proc <- which.min(pmax(proc_ready, curr_time))
      proc_ready[ready_proc] <- curr_time + costs[task]
      schedule[task,] <- c(task, ready_proc, curr_time,
                           proc_ready[ready_proc])
      scheduled[task] <- TRUE
    }
  }
  schedule
}

MaxMin <- function(nb_processors, dag, costs = rep(1, gorder(dag))) {
  MinMin(nb_processors, dag, costs, maxmin = TRUE)
}

# Schedule with Earliest Finish Time and back-filling (insertion)
schedule_EFT <-function (nb_processors, dag, costs, task_order,
                         insertion) {
  require(igraph)
  require(purrr)
  # Get infos
  edges <- get.edgelist(dag)
  n <- gorder(dag)
  # Prepare final schedule
  schedule <- list()
  for (j in 1:nb_processors)
    schedule[[j]] <- list()
  schedule$finish_time <- rep(-1, n)
  for (i in task_order) {
    # Earliest start due to precedence
    stopifnot(all(schedule$finish_time[edges[edges[,2] == i,1]] != -1))
    pred_earliest <- max(schedule$finish_time[edges[edges[,2] == i,1]], 0)
    # Find best proc
    min_earliest_finish <- Inf
    for (j in 1:nb_processors) {
      # List of available slots (back-filling)
      slots <- c(schedule[[j]]$start, Inf) -
        pmax(c(0, schedule[[j]]$end), pred_earliest)
      if (insertion)
        fitting_slot <- which(costs[i] <= slots)[1]
      else
        fitting_slot <- length(slots)
      start_time <- max(c(0, schedule[[j]]$end)[fitting_slot],
                        pred_earliest)
      # Keep schedule if best so far
      if (start_time + costs[i] < min_earliest_finish) {
        min_earliest_finish <- start_time + costs[i]
        # Schedule on best proc
        best_schedule <- schedule
        best_schedule[[j]]$task <- c(head(schedule[[j]]$task,
                                          fitting_slot - .5), i,
                                     tail(schedule[[j]]$task,
                                          -(fitting_slot - .5)))
        best_schedule[[j]]$start <- c(head(schedule[[j]]$start,
                                           fitting_slot - .5),
                                      start_time,
                                      tail(schedule[[j]]$start,
                                           -(fitting_slot - .5)))
        best_schedule[[j]]$end <- c(head(schedule[[j]]$end,
                                         fitting_slot - .5),
                                    start_time + costs[i],
                                    tail(schedule[[j]]$end,
                                         -(fitting_slot - .5)))
        best_schedule$finish_time[i] <- start_time + costs[i]
      }
    }
    schedule <- best_schedule
  }
  stopifnot(sum(map_int(1:nb_processors,
                        ~ ifelse(is_empty(schedule[[.]]), 0L,
                                 length(schedule[[.]]$task)))) == n)
  # Add proc number and convert in data frame
  map(1:nb_processors, ~ c(schedule[[.]]["task"],
                           list(proc = rep(., length(schedule[[.]]$task))),
                           schedule[[.]]["start"], schedule[[.]]["end"])) %>%
    discard(~ length(.$proc) == 0) %>%
    map(bind_cols) %>%
    bind_rows
}

HEFT <- function(nb_processors, dag, costs = rep(1, gorder(dag))) {
  require(igraph)
  require(purrr)
  # Get infos
  edges <- get.edgelist(dag)
  n <- gorder(dag)
  # Compute bottom-level
  BL <- rep(NA, n)
  ord_topo <- topo_sort(dag)
  for (i in rev(ord_topo)) {
    successors <- edges[edges[,1] == i,2]
    BL[i] <- max(BL[successors], 0) + costs[i]
  }
  stopifnot(all(!is.na(BL)))
  # Schedule with Earliest Finish Time and back-filling (insertion)
  schedule_EFT(nb_processors, dag, costs, order(BL, decreasing = TRUE),
               insertion = TRUE)
}

HCPT <- function(nb_processors, dag, costs = rep(1, gorder(dag))) {
  require(igraph)
  # Get infos
  edges <- get.edgelist(dag)
  n <- gorder(dag)
  # Compute earliest and latest starting times
  AEST <- rep(NA, n)
  ALST <- rep(NA, n)
  ord_topo <- topo_sort(dag)
  for (i in ord_topo) {
    pred <- edges[edges[,2] == i,1]
    AEST[i] <- max(AEST[pred] + costs[pred], 0)
  }
  LB <- max(AEST)
  for (i in rev(ord_topo)) {
    succ <- edges[edges[,1] == i,2]
    ALST[i] <- min(ALST[succ], LB + costs[i]) - costs[i]
  }
  stopifnot(all(!is.na(AEST)) && all(!is.na(ALST)))
  # Compute critical nodes, stack them and generate list
  CN <- which(AEST == ALST)
  CN <- CN[order(ALST[CN])] # increasing rather than decreasing
                            # (apparent mistake from the article)
  tasks <- NULL
  while (length(tasks) != n) {
    curr_task <- CN[1]
    pred <- edges[edges[,2] == curr_task,1]
    if (all(pred %in% tasks)) {
      # All predecessors are in the list
      tasks <- c(tasks, CN[1])
      CN <- tail(CN, -1)
      # In case of multiple exit tasks (the article assumes a unique
      # exit task), other non-critical tasks are inserted in any order
      if (length(CN) == 0)
        CN <- which(!seq_len(n) %in% tasks)
    } else {
      unlisted <- pred[!pred %in% tasks]
      # Remove the previously added unlisted predecessors (not
      # mentioned in the article but seemingly necessary)
      CN <- CN[!CN %in% unlisted]
      # Push the unlisted predecessors to the stack (in any order as
      # it is not defined in the article)
      CN <- c(unlisted, CN)
    }
  }
  stopifnot(all(seq_len(n) %in% tasks) && length(tasks) == n &&
            length(CN) == 0)
  # Schedule with Earliest Finish Time
  schedule_EFT(nb_processors, dag, costs, tasks, insertion = FALSE)
}
