library(Rcpp)

stddev <- function(X) {
  m <- mean(X)
  sqrt(mean((X - m) ^ 2))
}

BFS_levelsR <- function(adj_mat) {
  require(purrr)
  levels <- diag(adj_mat)
  traversed_nodes <- diag(adj_mat)
  while (any(traversed_nodes == 0)) {
    ready_tasks <- colSums(adj_mat) == 0 & !traversed_nodes
    stopifnot(length(ready_tasks) != 0)
    levels[ready_tasks] <- max(levels) + 1
    adj_mat[ready_tasks,] <- 0
    traversed_nodes[ready_tasks] <- 1
  }
  levels
}

cppFunction("
// Documentation is on http://adv-r.had.co.nz/Rcpp.html and
// https://cran.r-project.org/web/packages/Rcpp/vignettes/Rcpp-quickref.pdf
NumericVector BFS_levels(NumericMatrix adj_mat) {
  int nrow = adj_mat.nrow(), ncol = adj_mat.ncol();
  NumericMatrix copy = clone(adj_mat);
  NumericVector levels(nrow);
  int tasks = 0;
  int level = 1;
  while (tasks < nrow) {
    for (int j = 0; j < ncol; j++)
      if (sum(copy(_, j)) == 0 && levels[j] == 0) {
        levels[j] = level;
        tasks++;
      }
    for (int j = 0; j < ncol; j++)
      if (levels[j] == level)
        copy(j, _) = 0 * copy(j, _);
    level++;
  }
  return levels;
}
")

compute_shape <- function(adj_mat) {
  require(dplyr)
  BFS_levels(adj_mat) %>%
    table %>%
    as.vector
}

compute_mass <- function(shape) {
  largest_sum <- 0
  current_sum <- 0
  for (m in shape) {
    if (m == 1)
      current_sum <- 0
    else
      current_sum <- current_sum + m
    largest_sum <- max(largest_sum, current_sum)
  }
  largest_sum / sum(shape)
}

edge_prop <- function(adj_mat) {
  indeg <- colSums(adj_mat)
  outdeg <- rowSums(adj_mat)
  list(m_edges = sum(adj_mat),
       deg_max = max(indeg + outdeg),
       deg_max_in = max(indeg),
       deg_max_out = max(outdeg),
       deg_min = min(indeg + outdeg),
       deg_mean = mean(indeg + outdeg),
       deg_sd = stddev(indeg + outdeg),
       deg_sd_in = stddev(indeg),
       deg_sd_out = stddev(outdeg))
}

node_prop <- function(adj_mat) {
  shape <- compute_shape(adj_mat)
  list(n_nodes = nrow(adj_mat),
       length = length(shape),
       shape_max = max(shape),
       shape_min = min(shape),
       shape_mean = mean(shape),
       shape_sd = stddev(shape),
       shape_1 = head(shape, 1),
       shape_k = tail(shape, 1),
       mass = compute_mass(shape))
}

DAG_reverse <- function(adj_mat) {
  n <- nrow(adj_mat)
  rev <- adj_mat
  for (i in seq_len(n))
    for (j in seq_len(n))
      rev[i,j] <- adj_mat[(n - j + 1),(n - i + 1)]
  rev
}

DAG_transitive_reduce_slow <- function(adj_mat) {
  # Reorder matrix
  mat_ord <- order(BFS_levels(adj_mat))
  adj_mat <- adj_mat[mat_ord,mat_ord]

  longest_path <- function(adj_mat, i, j) {
    children <- list(i)
    longest <- 0
    while (TRUE) {
      current <- children[[length(children)]]
      next_children <- NULL
      for (i in current)
        next_children <- c(next_children, which(adj_mat[i,] == 1))
      next_children <- unique(next_children)
      if (j %in% next_children)
        longest <- length(children)
      if (length(next_children) == 0)
        break
      children[[length(children) + 1]] <- next_children
    }
    longest
  }
  n <- nrow(adj_mat)
  for (i in 1:(n - 1))
    for (j in (i + 1):n)
      if (adj_mat[i,j] == 1 && longest_path(adj_mat, i, j) > 1)
        adj_mat[i,j] <- 0
  adj_mat
}

DAG_transitive_reduceR <- function(adj_mat) {
  # Reorder matrix
  mat_ord <- order(BFS_levels(adj_mat))
  adj_mat <- adj_mat[mat_ord,mat_ord]

  n <- nrow(adj_mat)
  distances <- (diag(nrow = n) - 1) * n
  for (i in seq_len(n))
    for (j in seq_len(i - 1))
      if (adj_mat[j,i] == 1)
        distances[,i] <- pmax(distances[,j] + 1, distances[,i])
  for (i in seq_len(n - 1))
    for (j in (i + 1):n)
      if (adj_mat[i,j] == 1 && distances[i,j] > 1)
        adj_mat[i,j] <- 0
  adj_mat
}

cppFunction("
NumericMatrix DAG_transitive_reduce(NumericMatrix adj_mat) {
  int nrow = adj_mat.nrow(), ncol = adj_mat.ncol();

  NumericMatrix copy = clone(adj_mat);
  NumericVector idx(nrow);
  NumericVector selected(nrow);
  idx = idx - 1;
  int tasks = 0;
  while (tasks < nrow) {
    for (int j = 0; j < ncol; j++)
      if (sum(copy(_, j)) == 0 && selected[j] != 1) {
        selected[j] = 1;
        idx[tasks] = j;
        copy(j, _) = 0 * copy(j, _);
        tasks++;
      }
  }

  NumericMatrix dist(nrow, ncol);
  for (int i = 0; i < ncol; i++) {
    for (int j = 0; j < nrow; j++)
      dist(idx[j], idx[i]) = - nrow;
    dist(idx[i], idx[i]) = 0;
    for (int j = 0; j < i; j++)
      if (adj_mat(idx[j], idx[i]) == 1)
        dist(_, idx[i]) = pmax(dist(_, idx[j]) + 1, dist(_, idx[i]));
  }

  NumericMatrix out(nrow, ncol);
  for (int i = 0; i < nrow; i++) {
    for (int j = 0; j < ncol; j++) {
      if (adj_mat(idx[i], idx[j]) == 1 && dist(idx[i], idx[j]) == 1)
        out(idx[i], idx[j]) = 1;
      else
        out(idx[i], idx[j]) = 0;
    }
  }
  return out;
}
")

DAG_prop <- function(adj_mat) {
  prop_rev <- node_prop(DAG_reverse(adj_mat))
  prop_red <- edge_prop(DAG_transitive_reduce(adj_mat))
  names(prop_rev) <- paste0(names(prop_rev), "_rev")
  names(prop_red) <- paste0(names(prop_red), "_red")
  c(edge_prop(adj_mat), node_prop(adj_mat), prop_rev, prop_red)
}
